#if !defined(WINCOMPAT_INCLUDED) && !defined(PLATFORM_WINDOWS) && !defined(WIN32) && !defined(WINDOWS) && !defined(__WIN32__)
#define WINCOMPAT_INCLUDED

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <screen/screen.h>
#include <bps/screen.h>
#include <bps/bps.h>
#include <bps/virtualkeyboard.h>
#include <sys/keycodes.h>

#ifndef TRUE
  #define TRUE 1
#endif
#ifndef FALSE
  #define FALSE 0
#endif

#define _kbhit kbhit
#define _getch getch
#define stricmp strcasecmp
#define strnicmp strncasecmp

#define Sleep(x) usleep((x)*1000)

static int inited=0;

static int screen_val;

static screen_event_t screen_event;
static screen_context_t screen_context;

static void tcatexit()
{
    //Stop requesting events from libscreen
    screen_stop_events(screen_context);

    //Shut down BPS library for this process
    bps_shutdown();

    //Destroy libscreen context
    screen_destroy_context(screen_context);
}

static void init()
{
    const int usage = SCREEN_USAGE_NATIVE;

    screen_context_t screen_ctx;
    screen_window_t screen_win;
    screen_buffer_t screen_buf = NULL;
    int rect[4] = { 0, 0, 0, 0 };

    /* Setup the window */
    screen_create_context(&screen_ctx, 0);
    screen_create_window(&screen_win, screen_ctx);
    screen_set_window_property_iv(screen_win, SCREEN_PROPERTY_USAGE, &usage);
    screen_create_window_buffers(screen_win, 1);

    screen_get_window_property_pv(screen_win, SCREEN_PROPERTY_RENDER_BUFFERS, (void **)&screen_buf);
    screen_get_window_property_iv(screen_win, SCREEN_PROPERTY_BUFFER_SIZE, rect+2);

    /* Fill the screen buffer with blue */
    int attribs[] = { SCREEN_BLIT_COLOR, 0xff0000ff, SCREEN_BLIT_END };
    screen_fill(screen_ctx, screen_buf, attribs);
    screen_post_window(screen_win, screen_buf, 1, rect, 0);

    /* Signal bps library that navigator and screen events will be requested */
    bps_initialize();
    screen_request_events(screen_ctx);

    virtualkeyboard_show();
	
	atexit(tcatexit);
}

static inline int kbhit()
{
    int rc;
	bps_event_t *event = NULL;

	if (!inited)
	{
		inited = 1;
		init();
	}

	for (;;) 
	{
		rc = bps_get_event(&event, 0);

		if (event && rc == BPS_SUCCESS)
		{
			int domain = bps_event_get_domain(event);

			if (domain == screen_get_domain()) 
			{
				screen_event = screen_event_get_event(event);

				screen_get_event_property_iv(screen_event, SCREEN_PROPERTY_TYPE, &screen_val);
				
				if (screen_val == SCREEN_EVENT_KEYBOARD)
				{
					return TRUE;
				}
			}
		} 
		else 
		{
			break;
		}
	}
	return FALSE;
}

static inline int getch()
{					
    static bool pressed = false;
    if (screen_val & KEY_DOWN && !pressed)
    {
        screen_get_event_property_iv(screen_event, SCREEN_PROPERTY_KEY_SYM, &screen_val);
		pressed = !pressed;
		return screen_val;
	}
	
	pressed = !pressed;

	return 0xDeadBeef;	 
}

#endif
